#!/usr/bin/env python3

"""
@author Irumi Sugimori
Time-stamp: <2023-07-01 09:19:46 irumisugimori>
"""

import sys
import getopt
import re
import os
from util_col2asp import *


def _usage(msg=""):
    if msg != "":
        print(msg)
        print("")
    print(f"Usage: python3 {sys.argv[0]} <options> col_file")
    print(f"    -h --help               : show this help")
    print(f"    -p --partition          : use graph partition")
    exit()

class Config:
    def __init__(self):
        self.flag_p = False


config = Config()


"""
main
"""

def main():
    try:
        opts, args = getopt.getopt(sys.argv[1:], "hp", ["help", "partition"])
        for opt, arg in opts:
            if opt in ("-h", "--help"):
                _usage()
            elif opt in ("-p", "--partition"):
                config.flag_p = True
        if len(args) != 1:
            _usage("please input col-file.")

    except getopt.GetoptError as err:
        _usage(str(err))

    col_file = args[0]
    nodes = []
    edges = []
    start = -1
    goal = -1
    max_length = -1
    with open(col_file) as cf:
        for line in cf:
            p = re.match(r"^p\sedge\s(\d+)\s(\d+)", line)
            if p is not None:
                nodes = [i for i in range(1,int(p.group(1))+1)]
                continue
            e = re.match(r"^e\s(\d+)\s(\d+)", line)
            if e is not None:
                edges.append((int(e.group(1)), int(e.group(2))))
                continue
            l = re.match(r"^l\s(\d+)", line)
            if l is not None:
                max_length = int(l.group(1))
                continue
            t = re.match(r"^t\s(\d+)\s(\d+)", line)
            if t is not None:
                start = int(t.group(1))
                goal = int(t.group(2))
                continue

    print_fact_num_of_nodes(nodes)
    print_fact_num_of_edges(edges)
    print_fact_node(nodes)
    print_fact_edge(edges)
    print_fact_max_length(max_length)
    if start > 0:
        print_fact_terminal(start, goal)
    print_fact_distance(nodes, edges)
    print_fact_deadend_node(nodes, edges, start, goal)
    if config.flag_p:
        print_fact_partition(nodes, edges)

if __name__ == "__main__":
    main()

