#!/usr/bin/env python3

"""
@author Shuji Kosuge

"""

import sys
import getopt
import re
import numpy as np


class Config:
    def __init__(self, verbose=False):
        self.verbose = verbose
        # self.random_walk_seed = None
        self.random_walk_seed = 0# 再現性を出すために固定
        self.random_walk_count = 2000
        self.random_walk_life = 10
        self.classification_percentage = 0.4


config = Config()


def _usage(msg=""):
    if msg != "":
        print(f"{msg}\n")
    print(f"Usage: python3 {sys.argv[0]} <options> file")
    print(f"    -h --help               : show this help")
    print(f"    -v --verbose            : show detail")
    print(f"    -s --seed <int>         : select seed for random walk. (default: {config.random_walk_seed})")
    print(f"    -c --count <int>        : Repetition Count for random walk. (default: {config.random_walk_count})")
    print(f"    -l --life <int>         : random walk life. (default: {config.random_walk_life})")
    # print(f"    -p --percentage <float> : graph classification percentage. (default: {config.classification_percentage})")
    exit()


def random_walk(start, adjacent_vertex, life, seed=None):
    if seed is not None:
        np.random.seed(seed)
    path = []
    v = start
    for l in range(life):
        if len(adjacent_vertex[v]) == 0:
            continue
        n = np.random.randint(0, len(adjacent_vertex[v]))
        v = adjacent_vertex[v][n]
        path.append(v)

    return path


def graph_division(nodes, edges, seed=config.random_walk_seed, count=config.random_walk_count, life=config.random_walk_life):
    adjacent_vertex = {}
    for n in nodes:
        adjacent_vertex[n] = []
        for e in edges:
            if e[0] == n:
                adjacent_vertex[n].append(e[1])
            elif e[1] == n:
                adjacent_vertex[n].append(e[0])

    group = {}
    label_num = 0
    label = 0
    for n in nodes:
        visit_count = {}
        for nn in nodes:
            visit_count[nn] = 0
        for i in range(count):
            if seed is not None:
                seed = seed + i
            rw_result = random_walk(n, adjacent_vertex, life, seed)
            for r in rw_result:
                visit_count[r] += 1

        if n in group:
            if visit_count[n] > group[n][1]:
                label_num += 1
                label = label_num
            else:
                label = group[n][0]

        for nn in nodes:
            # if visit_count[n-1] > config.random_walk_count * config.classification_percentage:
            if not (nn in group):
                group[nn] = [label, visit_count[nn]]
            elif visit_count[nn] > group[nn][1]:
                group[nn] = [label, visit_count[nn]]

    result = {}
    group_num = []
    for g in group:
        result[g] = group[g][0]
        group_num.append(group[g][0])
    return list(set(group_num)), result


def make2atom(group):
    for g in group:
        print(f"partition({g}, {group[g]}).")


def main():
    global config
    try:
        opts, args = getopt.getopt(sys.argv[1:], "hvs:c:l:", ["help", "verbose", "seed", "count", "life"])

        for opt, arg in opts:
            if opt in ("-h", "--help"):
                _usage()
            elif opt in ("-v", "--verbose"):
                config.verbose = True
            elif opt in ("-s", "--seed"):
                config.random_walk_seed = int(arg)
            elif opt in ("-c", "--count"):
                config.random_walk_count = int(arg)
            elif opt in ("-l", "--life"):
                config.random_walk_life = int(arg)
            # elif opt in ("-p", "--percentage"):
            #     config.classification_percentage = float(arg)

        if len(args) != 1:
            _usage("please input file.")

    except getopt.GetoptError as err:
        _usage(str(err))

    col_file = args[0]
    nodes = []
    edges = []
    # start = -1
    # goal = -1
    # max_length = -1
    with open(col_file) as cf:
        for line in cf:
            p = re.match(r"^p\sedge\s(\d+)\s(\d+)", line)
            if p is not None:
                nodes = [i for i in range(1, int(p.group(1)) + 1)]
                continue
            e = re.match(r"^e\s(\d+)\s(\d+)", line)
            if e is not None:
                edges.append((int(e.group(1)), int(e.group(2))))
                continue
            # l = re.match(r"^l\s(\d+)", line)
            # if l is not None:
            #     max_length = int(l.group(1))
            #     continue
            # t = re.match(r"^t\s(\d+)\s(\d+)", line)
            # if t is not None:
            #     start = int(t.group(1))
            #     goal = int(t.group(2))
            #     continue
    num, group = graph_division(nodes, edges,config.random_walk_seed, config.random_walk_count,config.random_walk_life)

    if 10 >= len(num) >= 2:
        print(f"% partition: {len(num)} {num}")
        make2atom(group)



if __name__ == "__main__":
    main()
