#!/bin/bash


script_dir=$(dirname $0)
bin_dir=$script_dir/../bin
encoding_dir=$script_dir/../encoding

one_pair_encoder=$encoding_dir/onepair.lp
all_pairs_encoder=$encoding_dir/allpairs_directed.lp
col2asp=Col2Asp
col2asp_prepro=$bin_dir/col2asp_prepro.py

terminal_regex="^t [0-9]+ [0-9]+"
ans_regex="^Models       : ([0-9]+)"

usage() {
    echo "
usage:
    $0 [-h] [-v] [-t <int>] [-d <str>] <COLFILE>
" >&2
    exit 1
}

flag_v=false
thread=12
flag_d=false
directory=""

while getopts vt:d:h OPT; do
    case $OPT in
    v)
        flag_v=true
        ;;
    t)
	thread=$OPTARG
	;;
    d)
	flag_d=true
	directory=$OPTARG
	;;
    h | \?)
        usage && exit 1
        ;;
    esac
done

shift $((OPTIND - 1))

if [[ $# -ne 1 ]]; then
    usage
fi

if $flag_d; then
    mkdir -p $directory/lp
    mkdir -p $directory/log
    file_name=${1##*/}
    file_name=${file_name%%.*}
    lp_file=$directory/lp/$file_name\_$$.lp
    log_file=$directory/log/$file_name\_$$.log
else
    tmp_file=$(mktemp)
    lp_file=$tmp_file.lp
    log_file=$tmp_file.log
fi

function show_cmd {
    if $flag_v; then
        echo "c"
        echo "c ------------------------------------------------------------"
        echo "c CMD $1"
        echo "c ------------------------------------------------------------"
    fi
}

function verbose {
    if $flag_v; then
        sed "s/^/c /g" $1
    fi
}

function check_colfile {
    if [ ! -f $1 -o ! -s $1 ]; then
        echo "COLFILE ERROR: $1 is not exsist or is empty file."
        usage
    fi
}

encoder=$all_pairs_encoder
function check_terminal {
    while read line
    do
	if [[ $line =~ $terminal_regex ]]; then
	    encoder=$one_pair_encoder
	fi
    done < $1
}

function output_answer {
    while read line
    do
	if [[ $line =~ $ans_regex ]]; then
	    echo ${BASH_REMATCH[1]}
	    break
	fi
    done < $1
}

function rm_tmpfile {
  [[ -f "$1" ]] && rm -f "$1"
}

function finally {
    if ! $flag_d; then
	rm_tmpfile $tmp_file
	rm_tmpfile $lp_file
	rm_tmpfile $log_file
    fi
}

trap 'finally' EXIT
trap 'finally 2>/dev/null; exit 1' INT PIPE TERM

check_colfile $1

check_terminal $1

show_cmd "scala -classpath $bin_dir $col2asp $1 > $lp_file"
scala -classpath $bin_dir $col2asp $1 > $lp_file
show_cmd "python3 $col2asp_prepro $1 >> $lp_file"
python3 $col2asp_prepro $1 >> $lp_file
verbose $lp_file

show_cmd "clingo $encoder $lp_file -n 0 --quiet=2 -t $thread > $log_file"
clingo $encoder $lp_file -n 0 --quiet=2 -t $thread > $log_file 2>&1 &
pid=$!

wait
verbose $log_file
output_answer $log_file

