from graph_division import graph_division
import numpy as np
from scipy.sparse import csr_matrix
from scipy.sparse.csgraph import shortest_path


def print_message(*args):
    elems = [str(x) for x in args]
    print(" ".join(elems), flush=True)

    
def print_comment(*args):
    print_message('%', *args)

    
def _print_fact(predicate, *args):
    elems = [str(x) for x in args]
    print(predicate + "(" + ",".join(elems) + ")"+".")

    
def _deg(i, edges):
    return len([0 for e in edges if e[0] == i or e[1] == i])


def print_fact_num_of_nodes(nodes):
    _print_fact("num_of_nodes", len(nodes))

    
def print_fact_num_of_edges(edges):
    _print_fact("num_of_edges", len(edges))

    
def print_fact_node(nodes):
    _print_fact("node", f"1..{len(nodes)}")

    
def print_fact_edge(edges):
    for e in edges:
        _print_fact("edge", e[0], e[1])

        
def print_fact_max_length(max_length):
    _print_fact("max_length", max_length)

    
def print_fact_terminal(start, goal):
    _print_fact("terminal", start, goal)

    
def print_fact_distance(nodes, edges):
    n = len(nodes)
    edge = np.array([[e[0], e[1], 1] for e in edges], dtype=np.int64).T
    graph = csr_matrix((edge[2], (edge[:2] - 1)), (n, n))
    ans = shortest_path(graph, method='auto', directed=False)

    for i in range(1,n+1):
        for j in range(i,n+1):
            _print_fact("distance", i, j, int(ans[i-1][j-1])) 
                
def print_fact_deadend_node(nodes, edges, start, goal):
    if start < 0:
        return
    new_nodes = nodes.copy()
    new_edges = edges.copy()
    deadend_nodes = []
    check_nodes = nodes.copy()
    while check_nodes:
        tmp_nodes = [i for i in new_nodes if i not in check_nodes]
        tmp_edges = []
        next_check_nodes = []
        for i in check_nodes:
            if i not in [start, goal] and _deg(i,new_edges) <= 1:
                deadend_nodes.append(i)
            else:
                tmp_nodes.append(i)
        for e in new_edges:
            if e[0] in tmp_nodes and e[1] in tmp_nodes:
                tmp_edges.append(e)
            elif e[0] not in tmp_nodes:
                if e[1] not in next_check_nodes:
                    next_check_nodes.append(e[1])
            elif e[1] not in tmp_nodes:
                if e[0] not in next_check_nodes:
                    next_check_nodes.append(e[0])
        new_nodes = tmp_nodes.copy()
        new_edges = tmp_edges.copy()
        check_nodes = next_check_nodes.copy()
        
    for i in sorted(deadend_nodes):
        _print_fact("deadend_node", i)

        
def print_fact_partition(nodes, edges):
    num, partition = graph_division(nodes, edges)
    print_comment(f"number of partition: {len(num)}")
    if len(num) >= 2:
        for p in partition:
            _print_fact("partition", p, partition[p])
